'''
MQTT Packet
@author : Iwan Budi Kusnanto
'''
import sys

import nyamuk_const as NC
import nyamuk_net

class MqttPkt:
    """An mqtt packet."""
    def __init__(self):
        self.command = 0
        self.have_remaining = False
        self.remaining_count = 0
        self.mid = 0
        self.remaining_mult = 1
        self.remaining_length = 0
        self.packet_length = 0
        
        #len of packet to write
        self.to_process = 0
        
        #packet cursor
        self.pos = 0
        
        self.payload = None
        
        #self.next = None
    
    def dump(self):
        """Print packet content."""
        print "-----MqttPkt------"
        print "command = ", self.command
        print "have_remaining = ", self.have_remaining
        print "remaining_count = ", self.remaining_count
        print "mid = ", self.mid
        print "remaining_mult = ", self.remaining_mult
        print "remaining_length = ", self.remaining_length
        print "packet_length = ", self.packet_length
        print "to_process = ", self.to_process
        print "pos = ", self.pos
        print "payload = ", self.payload
        print "------------------"
        
    def alloc(self):
        """from _mosquitto_packet_alloc."""
        byte = 0
        remaining_bytes = bytearray(5)
        i = 0
        
        remaining_length = self.remaining_length
        
        self.payload = None
        self.remaining_count = 0
        loop_flag = True
        
        #self.dump()
        while loop_flag:
            byte = remaining_length % 128
            remaining_length = remaining_length / 128
            
            if remaining_length > 0:
                byte = byte | 0x80
                
            remaining_bytes[self.remaining_count] = byte
            self.remaining_count += 1
            
            if not (remaining_length > 0 and self.remaining_count < 5):
                loop_flag = False
        
        if self.remaining_count == 5:
            return NC.ERR_PAYLOAD_SIZE
        
        self.packet_length = self.remaining_length + 1 + self.remaining_count
        self.payload = bytearray(self.packet_length)
        
        self.payload[0] = self.command
        
        i = 0
        while i < self.remaining_count:
            self.payload[i+1] = remaining_bytes[i]
            i += 1
        
        self.pos = 1 + self.remaining_count
        
        return NC.ERR_SUCCESS
    
    def packet_cleanup(self):
        self.command = 0
        self.have_remaining = False
        self.remaining_count = 0
        self.remaining_mult = 1
        self.remaining_length = 0
        self.payload = None
        self.to_process = 0
        self.pos = 0
    
    def connect_build(self, nyamuk, keepalive, clean_session, retain = 0, dup = 0):
        """Build packet for CONNECT command."""
        will = 0
        byte = 0
        
        #payload len
        payload_len = 2 + len(nyamuk.client_id)
        if nyamuk.will is not None:
            will = 1
            payload_len = payload_len + 2 + len(nyamuk.will.topic) + 2 + nyamuk.will.payloadlen
        
        if nyamuk.username is not None:
            payload_len = payload_len + 2 + len(nyamuk.username)
            if nyamuk.password != None:
                payload_len = payload_len + 2 + len(nyamuk.password)
        
        self.command = NC.CMD_CONNECT
        self.remaining_length = 12 + payload_len
    
        rc = self.alloc()
        if rc != NC.ERR_SUCCESS:
            return rc
         
        #var header
        self.write_string(NC.PROTOCOL_NAME)
        self.write_byte(NC.PROTOCOL_VERSION)
        
        byte = (clean_session & 0x1) << 1
        
        if will:
            byte = byte | ((nyamuk.will.retain & 0x1) << 5) | ((nyamuk.will.qos & 0x3) << 3) | ((will & 0x1) << 2)
        
        if nyamuk.username is not None:
            byte = byte | 0x1 << 7
            if nyamuk.password is not None:
                byte = byte | 0x1 << 6
        
        self.write_byte(byte)
        self.write_uint16(keepalive)
        #payload
        self.write_string(nyamuk.client_id)
        
        if will:
            #TODO
            sys.exit(-1)
        if nyamuk.username is not None:
            self.write_string(nyamuk.username)
            if nyamuk.password is not None:
                self.write_string(nyamuk.password)
            
        nyamuk.keep_alive = keepalive
        
        return NC.ERR_SUCCESS
    
    def write_string(self, string):
        """Write a string to this packet."""
        self.write_uint16(len(string))
        self.write_bytes(string, len(string))
        
    def write_uint16(self, word):
        """Write 2 bytes."""
        self.write_byte(nyamuk_net.MOSQ_MSB(word))
        self.write_byte(nyamuk_net.MOSQ_LSB(word))
        
    def write_byte(self, byte):
        """Write one byte."""
        self.payload[self.pos] = byte
        self.pos = self.pos + 1
    
    def write_bytes(self, data, n):
        """Write n number of bytes to this packet."""
        for pos in xrange(0, n):
            self.payload[self.pos + pos] = data[pos]
            
        self.pos += n
    
    def read_byte(self):
        """Read a byte."""
        if self.pos + 1 > self.remaining_length:
            return NC.ERR_PROTOCOL, None
        
        byte = self.payload[self.pos]
        self.pos += 1
        
        return NC.ERR_SUCCESS, byte
    
    def read_uint16(self):
        """Read 2 bytes."""
        if self.pos + 2 > self.remaining_length:
            return NC.ERR_PROTOCOL
        msb = self.payload[self.pos]
        self.pos += 1
        lsb = self.payload[self.pos]
        self.pos += 1
        
        word = (msb << 8) + lsb
        
        return NC.ERR_SUCCESS, word
    
    def read_bytes(self, count):
        """Read count number of bytes."""
        if self.pos + count > self.remaining_length:
            return NC.ERR_PROTOCOL, None
        
        ba = bytearray(count)
        for x in xrange(0, count):
            ba[x] = self.payload[self.pos]
            self.pos += 1
        
        return NC.ERR_SUCCESS, ba
    
    def read_string(self):
        """Read string."""
        rc, length = self.read_uint16()
        
        if rc != NC.ERR_SUCCESS:
            return rc, None
        
        if self.pos + length > self.remaining_length:
            return NC.ERR_PROTOCOL, None
        
        ba = bytearray(length)
        if ba is None:
            return NC.ERR_NO_MEM, None
        
        for x in xrange(0, length):
            ba[x] = self.payload[self.pos]
            self.pos += 1
        
        return NC.ERR_SUCCESS, ba


"""
Nyamuk networking module.
Copyright(c) 2012 Iwan Budi Kusnanto
"""
import socket

def MOSQ_MSB(A):
    """get most significant byte."""
    return (( A & 0xFF00) >> 8)
    
def MOSQ_LSB(A):
    """get less significant byte."""
    return (A & 0x00FF)

def connect(sock, addr):
    """Connect to some addr."""
    try:
        sock.connect(addr)
    except socket.error as (_, msg):
        return (socket.error, msg)
    except socket.herror as (_, msg):
        return (socket.herror, str)
    except socket.gaierror as (_, msg):
        return (socket.gaierror, msg)
    except socket.timeout:
        return (socket.timeout, "timeout")
    
    return None
    
def read(sock, count):
    """Read from socket and return it's byte array representation.
    count = number of bytes to read
    """
    try:
        data = sock.recv(count)
    except socket.error as (_, msg):
        return data, (socket.error, msg)
    except socket.herror as (_, msg):
        return data, (socket.error, msg)
    except socket.gaierror as (_, msg):
        return data, (socket.gaierror, msg)
    except socket.timeout:
        return data, (socket.timeout, "timeout")
    
    ba_data = bytearray(data)
    return ba_data, None

def write(sock, payload):
    """Write payload to socket."""
    try:
        length = sock.send(payload)
    except socket.error as (_, msg):
        return -1, (socket.error, msg)
    except socket.herror as (_, msg):
        return -1, (socket.error, msg)
    except socket.gaierror as (_, msg):
        return -1, (socket.gaierror, msg)
    except socket.timeout:
        return -1, (socket.timeout, "timeout")
    
    return length, None

def setkeepalives(sock):
    """set sock to be keepalive socket."""
    sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)


'''
Nyamuk publisher example
copyright 2012 Iwan Budi Kusnanto
'''
import sys

from nyamuk import nyamuk
import nyamuk.nyamuk_const as NC
from nyamuk import event

def handle_connack(ev):
    print "CONNACK received"
    rc = ev.ret_code
    if rc == NC.CONNECT_ACCEPTED:
        print "\tconnection success"
    elif rc == 1:
        print "\tConnection refused : unacceptable protocol version"
    elif rc == 2:
        print "\tConnection refused : identifier rejected"
    elif rc == 3:
        print "\tConnection refused : broker unavailable"
    elif rc == 4:
        print "\tConnection refused : bad username or password"
    elif rc == 5:
        print "\tConnection refused : not authorized"
    else:
        print "\tConnection refused : unknown reason = ", rc
    
    return rc
    
def start_nyamuk(server, client_id, topic, payload):
    ny = nyamuk.Nyamuk(client_id, server = server)
    rc = ny.connect()
    if rc != NC.ERR_SUCCESS:
        print "Can't connect"
        sys.exit(-1)
    
    index = 0
    
    while rc == NC.ERR_SUCCESS:
        ev = ny.pop_event()
        if ev != None:
            if ev.type == NC.CMD_CONNACK:
                ret_code = handle_connack(ev)
                if ret_code == NC.CONNECT_ACCEPTED:
                    print "publishing payload"
                    ny.publish(topic, payload)
                
        rc = ny.loop()
    
if __name__ == '__main__':
    if len(sys.argv) != 5:
        print "usage   : python submq.py server name topic payload"
        print "example : python submq.py localhost sub-iwan teknobridges HaloMqtt"
        sys.exit(0)
        
    server = sys.argv[1]
    name = sys.argv[2]
    topic = sys.argv[3]
    payload = sys.argv[4]
    start_nyamuk(server, name, topic, payload)


'''
Nyamuk
Python MQTT Client Library
Copyright 2012 Iwan Budi Kusnanto <iwan.b.kusnanto@gmail.com>
'''
import time
import errno

from mqtt_pkt import MqttPkt
import nyamuk_const as NC
import nyamuk_net

MQTTCONNECT = 16# 1 << 4
class BaseNyamuk:
    """Base class of nyamuk."""
    def __init__(self, client_id, username, password,
                 server, port, keepalive):
        ''' Constructor '''
        self.client_id = client_id
        self.username = username
        self.password = password
        self.server = server
        self.port = port
        
        self.address = ""
        self.keep_alive = keepalive
        self.clean_session = 1
        self.state = NC.CS_NEW
        self.last_msg_in = time.time()
        self.last_msg_out = time.time()
        self.last_mid = 0
        
        #output packet queue
        self.out_packet = []
        
        #input packet queue
        self.in_packet = MqttPkt()
        self.in_packet.packet_cleanup()
        
        #networking
        self.sock = NC.INVALID_SOCKET
        
        self.will = None
        
        self.message_retry = NC.MESSAGE_RETRY
        self.last_retry_check = 0
        self.messages = None
        
        self.bridge = None
        
        #LOGGING Option:TODO
        self.log_priorities = -1
        self.log_destinations = -1
        
        self.host = None
        self.port = 1883
        
        #hack var
        self.as_broker = False
        
        #event list
        self.event_list = []
        
    def pop_event(self):
        """Pop an event from event_list."""
        if len(self.event_list) > 0:
            evt = self.event_list.pop(0)
            return evt
        return None
    
    def push_event(self, evt):
        """Add an event to event_list."""
        self.event_list.append(evt)
    
    def mid_generate(self):
        """Generate mid. TODO : check."""
        self.last_mid += 1
        if self.last_mid == 0:
            self.last_mid += 1
        return self.last_mid
    
    def packet_queue(self, pkt):
        """Enqueue packet to out_packet queue."""
        
        pkt.pos = 0
        pkt.to_process = pkt.packet_length
        
        self.out_packet.append(pkt)
        return NC.ERR_SUCCESS
    
    def packet_write(self):
        """Write packet to network."""
        bytes_written = 0
        
        if self.sock == NC.INVALID_SOCKET:
            return NC.ERR_NO_CONN, bytes_written
        
        while len(self.out_packet) > 0:
            pkt = self.out_packet[0]
            write_length, status = nyamuk_net.write(self.sock, pkt.payload)
            if write_length > 0:
                pkt.to_process -= write_length
                pkt.pos += write_length
                
                bytes_written += write_length
                
                if pkt.to_process > 0:
                    return NC.ERR_SUCCESS, bytes_written
            else:
                if status == errno.EAGAIN or status == errno.EWOULDBLOCK:
                    return NC.ERR_SUCCESS, bytes_written
                elif status == errno.ECONNRESET:
                    return NC.ERR_CONN_LOST, bytes_written
                else:
                    return NC.ERR_UNKNOWN, bytes_written
            
            """
            if pkt.command & 0xF6 == NC.CMD_PUBLISH and self.on_publish is not None:
                self.in_callback = True
                self.on_publish(pkt.mid)
                self.in_callback = False
            """
            
            #next
            del self.out_packet[0]
            
            #free data (unnecessary)
            
            self.last_msg_out = time.time()
            
        
        return NC.ERR_SUCCESS, bytes_written
    
    def packet_read(self):
        """Read packet from network."""
        bytes_received = 0
        
        if self.sock == NC.INVALID_SOCKET:
            return NC.ERR_NO_CONN
        
        if self.in_packet.command == 0:
            ba_data, status = nyamuk_net.read(self.sock, 1)
            readlen = len(ba_data)
            if readlen == 1:
                bytes_received += 1
                byte = ba_data[0]
                self.in_packet.command = byte
                
                if self.as_broker:
                    if self.bridge is None and self.state == NC.CS_NEW and (byte & 0xF0) != NC.CMD_CONNECT:
                        print "RETURN ERR_PROTOCOL"
                        return NC.ERR_PROTOCOL, bytes_received
            else:
                if readlen == 0:
                    return NC.ERR_CONN_LOST, bytes_received
                if status == errno.EAGAIN or status == errno.EWOULDBLOCK:
                    return NC.ERR_SUCCESS, bytes_received
                else:
                    if status == errno.ECONNRESET:
                        return NC.ERR_CONN_LOST, bytes_received
                    else:
                        return NC.ERR_UNKNOWN, bytes_received
        
        if not self.in_packet.have_remaining:
            loop_flag = True
            while loop_flag:
                ba_data, status = nyamuk_net.read(self.sock, 1)
                readlen = len(ba_data)
                byte = ba_data[0]
                if readlen == 1:
                    bytes_received += 1
                    self.in_packet.remaining_count += 1
                    if self.in_packet.remaining_count > 4:
                        return NC.ERR_PROTOCOL, bytes_received
                    
                    self.in_packet.remaining_length += (byte & 127) * self.in_packet.remaining_mult
                    self.in_packet.remaining_mult *= 128
                else:
                    if readlen == 0:
                        return NC.ERR_CONN_LOST, bytes_received
                
                if (byte & 128) == 0:
                    loop_flag = False
            
            if self.in_packet.remaining_length > 0:
                self.in_packet.payload = bytearray(self.in_packet.remaining_length)
                if self.in_packet.payload is None:
                    return NC.ERR_NO_MEM, bytes_received
                self.in_packet.to_process = self.in_packet.remaining_length
            
            self.in_packet.have_remaining = True
        
        if self.in_packet.to_process > 0:
            ba_data, status = nyamuk_net.read(self.sock, self.in_packet.to_process)
            readlen = len(ba_data)
            if readlen > 0:
                bytes_received += readlen
                for idx in xrange(0, readlen):
                    self.in_packet.payload[self.in_packet.pos] = ba_data[idx]
                    self.in_packet.pos += 1
                    self.in_packet.to_process -= 1
            else:
                if status == errno.EAGAIN or status == errno.EWOULDBLOCK:
                    return NC.ERR_SUCCESS, bytes_received
                else:
                    if status == errno.ECONNRESET:
                        return NC.ERR_CONN_LOST, bytes_received
                    else:
                        return NC.ERR_UNKNOWN, bytes_received
        
        #all data for this packet is read
        self.in_packet.pos = 0
        
        ret = self.packet_handle()
        
        self.in_packet.packet_cleanup()
        
        self.last_msg_in = time.time()
        
        return ret, bytes_received
                
    def socket_close(self):
        """Close our socket."""
        if self.sock != NC.INVALID_SOCKET:
            self.sock.close()
        self.sock = NC.INVALID_SOCKET
        
    def build_publish_pkt(self, mid, topic, payload, qos, retain, dup):
        """Build PUBLISH packet."""
        pkt = MqttPkt()
        payloadlen = len(payload)
        packetlen = 2 + len(topic) + payloadlen
        
        if qos > 0:
            packetlen += 2
        
        pkt.mid = mid
        pkt.command = NC.CMD_PUBLISH | ((dup & 0x1) << 3) | (qos << 1) | retain
        pkt.remaining_length = packetlen
        
        ret = pkt.alloc()
        if ret != NC.ERR_SUCCESS:
            return ret, None
        
        #variable header : Topic String
        pkt.write_string(topic)
        
        if qos > 0:
            pkt.write_uint16(mid)
        
        #payloadlen
        if payloadlen > 0:
            pkt.write_bytes(payload, payloadlen)
        
        return NC.ERR_SUCCESS, pkt
    
    def send_simple_command(self, cmd):
        """Send simple mqtt commands."""
        pkt = MqttPkt()
        
        pkt.command = cmd
        pkt.remaining_length = 0
        
        ret = pkt.alloc()
        if ret != NC.ERR_SUCCESS:
            return ret
        
        return self.packet_queue(pkt)


"""Nyamuk event."""
import nyamuk_const as NC

class BaseEvent:
    """Event Base Class."""
    def __init__(self, tipe):
        self.type = tipe

class EventConnack(BaseEvent):
    """CONNACK received."""
    def __init__(self, ret_code):
        BaseEvent.__init__(self, NC.CMD_CONNACK)
        self.ret_code = ret_code

class EventPublish(BaseEvent):
    """PUBLISH received."""
    def __init__(self, msg):
        BaseEvent.__init__(self, NC.CMD_PUBLISH)
        self.msg = msg

class EventSuback(BaseEvent):
    """SUBACK received."""
    def __init__(self, mid, granted_qos):
        BaseEvent.__init__(self, NC.CMD_SUBACK)
        self.mid = mid
        self.granted_qos = granted_qos



"""
Nyamuk : Python MQTT Client library
Copyright 2012 Iwan Budi Kusnanto
"""
import socket
import select
import time
import sys
import logging

import base_nyamuk
import nyamuk_const as NC
from mqtt_pkt import MqttPkt
from nyamuk_msg import NyamukMsgAll
import nyamuk_net
import event

class Nyamuk(base_nyamuk.BaseNyamuk):
    """Nyamuk mqtt client class."""
    def __init__(self, client_id, username = None, password = None,
                 server = "localhost", port = 1883, keepalive = NC.KEEPALIVE_VAL,
                 log_level = logging.DEBUG):
        base_nyamuk.BaseNyamuk.__init__(self, client_id, username, password,
                                        server, port, keepalive)
        
        #logging
        self.logger = logging.getLogger(client_id)
        self.logger.setLevel(log_level)
        
        ch = logging.StreamHandler()
        ch.setLevel(log_level)
        
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        
        ch.setFormatter(formatter)
        
        self.logger.addHandler(ch)
        
    def loop(self, timeout = 1):
        """Main loop."""
        rlist = [self.sock]
        wlist = []
        if len(self.out_packet) > 0:
            wlist.append(self.sock)
        
        to_read, to_write, _ = select.select(rlist, wlist, [], timeout)
        
        if len(to_read) > 0:
            ret, _ = self.loop_read()
            if ret != NC.ERR_SUCCESS:
                return ret
        
        if len(to_write) > 0:
            ret, _ = self.loop_write()
            if ret != NC.ERR_SUCCESS:
                return ret
            
        self.loop_misc()
        
        return NC.ERR_SUCCESS
    
    def loop_read(self):
        """Read loop."""
        ret, bytes_received = self.packet_read()
        return ret, bytes_received
    
    def loop_write(self):
        """Write loop."""
        ret, bytes_written =  self.packet_write()
        return ret, bytes_written
    
    def loop_misc(self):
        """Misc loop."""
        self.check_keepalive()
        if self.last_retry_check + 1  < time.time():
            pass
        return NC.ERR_SUCCESS
    
    def check_keepalive(self):
        """Send keepalive/PING if necessary."""
        if self.sock != NC.INVALID_SOCKET and time.time() - self.last_msg_out >= self.keep_alive:
            if self.state == NC.CS_CONNECTED:
                self.send_pingreq()
            else:
                self.socket_close()
    
    def packet_handle(self):
        """Incoming packet handler dispatcher."""
        cmd = self.in_packet.command & 0xF0
        
        if cmd == NC.CMD_CONNACK:
            return self.handle_connack()
        elif cmd == NC.CMD_PINGRESP:
            return self.handle_pingresp()
        elif cmd == NC.CMD_PUBLISH:
            return self.handle_publish()
        elif cmd == NC.CMD_PUBACK:
            print "Received PUBACK"
            sys.exit(-1)
        elif cmd == NC.CMD_PUBREC:
            print "Received PUBREC"
            sys.exit(-1)
        elif cmd == NC.CMD_PUBREL:
            print "Received PUBREL"
            sys.exit(-1)
        elif cmd == NC.CMD_PUBCOMP:
            print "Received PUBCOMP"
            sys.exit(-1)
        elif cmd == NC.CMD_SUBSCRIBE:
            sys.exit(-1)
        elif cmd == NC.CMD_SUBACK:
            return self.handle_suback()
        elif cmd == NC.CMD_UNSUBSCRIBE:
            print "Received UNSUBSCRIBE"
            sys.exit(-1)
        elif cmd == NC.CMD_UNSUBACK:
            print "Received UNSUBACK"
            sys.exit(-1)
        else:
            self.logger.warning("Unknown protocol. Cmd = %d", cmd)
            return NC.ERR_PROTOCOL
    
    def connect(self, clean_session = 1):
        """Connect to server."""
        self.clean_session = clean_session
        
        #CONNECT packet
        pkt = MqttPkt()
        pkt.connect_build(self, self.keep_alive, clean_session)
        
        #create socket
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        nyamuk_net.setkeepalives(self.sock)
        
        self.logger.info("Connecting to server ....%s", self.server)
        err = nyamuk_net.connect(self.sock,(self.server, self.port))
        
        if err != None:
            self.logger.error(err[1])
            return NC.ERR_UNKNOWN
        
        #set to nonblock
        self.sock.setblocking(0)
        
        return self.packet_queue(pkt)
    
    def disconnect(self):
        """Disconnect from server."""
        self.logger.info("DISCONNECT")
        if self.sock == NC.INVALID_SOCKET:
            return NC.ERR_NO_CONN
        self.state = NC.CS_DISCONNECTING
        
        ret = self.send_disconnect()
        self.socket_close()
        return ret
    
    def subscribe(self, topic, qos):
        """Subscribe to some topic."""
        if self.sock == NC.INVALID_SOCKET:
            return NC.ERR_NO_CONN
        
        self.logger.info("SUBSCRIBE: %s", topic)
        return self.send_subscribe(False, topic, qos)
    
    def send_disconnect(self):
        """Send disconnect command."""
        return self.send_simple_command(NC.CMD_DISCONNECT)
        
    def send_subscribe(self, dup, topic, qos):
        """Send subscribe COMMAND to server."""
        pkt = MqttPkt()
        
        pktlen = 2 + 2 + len(topic) + 1
        pkt.command = NC.CMD_SUBSCRIBE | (dup << 3) | (1 << 1)
        pkt.remaining_length = pktlen
        
        ret = pkt.alloc()
        if ret != NC.ERR_SUCCESS:
            return ret
        
        #variable header
        mid = self.mid_generate()
        pkt.write_uint16(mid)
        
        #payload
        pkt.write_string(topic)
        pkt.write_byte(qos)
        
        return self.packet_queue(pkt)
    
    def publish(self, topic, payload = None, qos = 0, retain = False):
        """Publish some payload to server."""
        #print "PUBLISHING (",topic,"): ", payload
        payloadlen = len(payload)
        if topic is None or qos < 0 or qos > 2:
            print "PUBLISH:err inval"
            return NC.ERR_INVAL
        
        #payloadlen <= 250MB
        if payloadlen > (250 * 1024 * 1204):
            self.logger.error("PUBLISH:err payload len:%d", payloadlen)
            return NC.ERR_PAYLOAD_SIZE
        
        #wildcard check : TODO
        mid = self.mid_generate()
        
        if qos == 0:
            return self.send_publish(mid, topic, payload, qos, retain, False)
        else:
            self.logger.error("Unsupport QoS=", qos)
    
    def handle_connack(self):
        """Handle incoming CONNACK command."""
        self.logger.info("CONNACK reveived")
        ret, _ = self.in_packet.read_byte()
        if ret != NC.ERR_SUCCESS:
            self.logger.error("error read byte")
            return ret
        
        ret, result = self.in_packet.read_byte()
        if ret != NC.ERR_SUCCESS:
            return ret
        
        evt = event.EventConnack(ret)
        self.push_event(evt)
        
        if result == NC.CONNECT_ACCEPTED:
            self.state = NC.CS_CONNECTED
            return NC.ERR_SUCCESS
        
        elif result >= 1 and result <= 5:
            return NC.ERR_CONN_REFUSED
        else:
            return NC.ERR_PROTOCOL
        
    def handle_pingresp(self):
        """Handle incoming PINGRESP packet."""
        self.logger.debug("PINGRESP received")
        return NC.ERR_SUCCESS
    
    def handle_suback(self):
        """Handle incoming SUBACK packet."""
        self.logger.info("SUBACK received")
        
        ret, mid = self.in_packet.read_uint16()
        
        if ret != NC.ERR_SUCCESS:
            return ret
        
        qos_count = self.in_packet.remaining_length - self.in_packet.pos
        granted_qos = bytearray(qos_count)
        
        if granted_qos is None:
            return NC.ERR_NO_MEM
        
        i = 0
        while self.in_packet.pos < self.in_packet.remaining_length:
            ret, byte = self.in_packet.read_byte()
            
            if ret != NC.ERR_SUCCESS:
                granted_qos = None
                return ret
            
            granted_qos[i] = byte
            
            i += 1
        
        evt = event.EventSuback(mid, granted_qos)
        self.push_event(evt)
        
        granted_qos = None
        
        return NC.ERR_SUCCESS
    
    def send_pingreq(self):
        """Send PINGREQ command to server."""
        self.logger.debug("SEND PINGREQ")
        self.send_simple_command(NC.CMD_PINGREQ)
    
    def handle_publish(self):
        """Handle incoming PUBLISH packet."""
        self.logger.debug("PUBLISH received")
        
        header = self.in_packet.command
        
        message = NyamukMsgAll()
        message.direction = NC.DIRECTION_IN
        message.dup = (header & 0x08) >> 3
        message.msg.qos = (header & 0x06) >> 1
        message.msg.retain = (header & 0x01)
        
        ret, ba_data = self.in_packet.read_string()
        message.msg.topic = ba_data.decode()
        
        if ret != NC.ERR_SUCCESS:
            return ret
        
        #fix_sub_topic TODO
        if message.msg.qos > 0:
            ret, word = self.in_packet.read_uint16()
            message.msg.mid = word
            if ret != NC.ERR_SUCCESS:
                return ret
        
        message.msg.payloadlen = self.in_packet.remaining_length - self.in_packet.pos
        
        if message.msg.payloadlen > 0:
            ret, message.msg.payload = self.in_packet.read_bytes(message.msg.payloadlen)
            if ret != NC.ERR_SUCCESS:
                return ret
        
        self.logger.debug("Received PUBLISH(dup = %d,qos=%d,retain=%s", message.dup, message.msg.qos, message.msg.retain)
        self.logger.debug("\tmid=%d, topic=%s, payloadlen=%d", message.msg.mid, message.msg.topic, message.msg.payloadlen)
        
        message.timestamp = time.time()
        
        qos = message.msg.qos
        
        if qos == 0:
            evt = event.EventPublish(message.msg)
            self.push_event(evt)

            return NC.ERR_SUCCESS
        
        elif qos == 1 or qos == 2:
            self.logger.error("handle_publish. Unsupported QoS = 1 or QoS = 2")
            sys.exit(-1)
        else:
            return NC.ERR_PROTOCOL
        
        return NC.ERR_SUCCESS
    
    def send_publish(self, mid, topic, payload, qos, retain, dup):
        """Send PUBLISH."""
        self.logger.debug("Send PUBLISH")
        if self.sock == NC.INVALID_SOCKET:
            return NC.ERR_NO_CONN
        return self._do_send_publish(mid, topic, payload, qos, retain, dup)
    
    def _do_send_publish(self, mid, topic, payload, qos, retain, dup):
        ret, pkt = self.build_publish_pkt(mid, topic, payload, qos, retain, dup)
        if ret != NC.ERR_SUCCESS:
            return ret
        
        return self.packet_queue(pkt)


'''
Nyamuk subscriber example
Copyright Iwan Budi Kusnanto
'''
import sys
import logging

from nyamuk import nyamuk
from nyamuk import nyamuk_const as NC
from nyamuk import event

def handle_connack(ev):
    print "CONNACK received"
    rc = ev.ret_code
    if rc == 0:
        print "\tconnection success"
    elif rc == 1:
        print "\tConnection refused : unacceptable protocol version"
    elif rc == 2:
        print "\tConnection refused : identifier rejected"
    elif rc == 3:
        print "\tConnection refused : broker unavailable"
    elif rc == 4:
        print "\tConnection refused : bad username or password"
    elif rc == 5:
        print "\tConnection refused : not authorized"
    else:
        print "\tConnection refused : unknown reason = ", rc
    
    return rc
        
def handle_publish(ev):
    msg = ev.msg
    print "PUBLISH_RECEIVED"
    print "\ttopic : " + msg.topic
    if msg.payload is not None:
        print "\tpayload : " + msg.payload

def handle_suback(ev):
    print "SUBACK RECEIVED"
    print "\tQOS Count = ", len(ev.granted_qos)
    print "\tMID = ", ev.mid
        
def start_nyamuk(server, client_id, topic, username = None, password = None):
    ny = nyamuk.Nyamuk(client_id, username, password, server)
    rc = ny.connect()
    if rc != NC.ERR_SUCCESS:
        print "Can't connect"
        sys.exit(-1)
    
    rc = ny.subscribe(topic, 0)
    
    while rc == NC.ERR_SUCCESS:
        ev = ny.pop_event()
        if ev != None:
            if ev.type == NC.CMD_CONNACK:
                handle_connack(ev)
            elif ev.type == NC.CMD_PUBLISH:
                handle_publish(ev)
            elif ev.type == NC.CMD_SUBACK:
                handle_suback(ev)
        rc = ny.loop()
    
if __name__ == '__main__':
    if len(sys.argv) != 4:
        print "usage    : python submq.py server client_id topic"
        print "example  : python submq.py localhost sub-iwan teknobridges"
        sys.exit(0)
        
    server = sys.argv[1]
    client_id = sys.argv[2]
    topic = sys.argv[3]
    start_nyamuk(server, client_id, topic)


"""
Copyright(c)2012 Iwan Budi Kusnanto
"""
import time

import nyamuk_const as NC

class NyamukMsg:
    """Nyamuk message."""
    def __init__(self):
        self.mid = 0
        self.topic = None
        self.payload = None
        self.payloadlen = -1
        self.qos = -1
        self.retain = False
        
class NyamukMsgAll:
    
    def __init__(self):
        #next
        self.timestamp = time.time()
        self.direction = NC.DIRECTION_NONE
        self.state = NC.MS_INVALID
        self.dup = False
        self.msg = NyamukMsg()


import os, sys
from setuptools import Command, setup

long_description = open('README').read()

setup(
    author = 'Iwan Budi Kusnanto',
    url = 'https://github.com/iwanbk/nyamuk',
    author_email ='iwan.b.kusnanto@gmail.com',
    version ='0.1',
    install_requires = [],
    packages = ['nyamuk'],
    name = 'nyamuk',
    description = 'Python MQTT Client Library',
    long_description = long_description,
    license = "BSD",
)



import sys
import logging

from nyamuk import nyamuk
from nyamuk import nyamuk_const as NC

the_topic = ""
def on_connect(obj, rc):
    if rc == 0:
        print "on_connect callback : connection success"
        #print "subscribing to topic = ", the_topic
        #mid = 1
        #rc = obj.subscribe(mid, topic, 0)
    elif rc == 1:
        print "Connection refused : unacceptable protocol version"
    elif rc == 2:
        print "Connection refused : identifier rejected"
    elif rc == 3:
        print "Connection refused : broker unavailable"
    elif rc == 4:
        print "Connection refused : bad username or password"
    elif rc == 5:
        print "Connection refused : not authorized"
    else:
        print "Connection refused : unknown reason = ", rc
    
def on_message(nyamuk, msg):
    print "--- message --"
    print "topic : " + msg.topic
    if msg.payload is not None:
        print "payload : " + msg.payload

def on_subscribe(nyamuk, mid, granted_qos):
    print "On Subscribe"
    print "\tQOS Count = ", len(granted_qos)
    print "\tMID = ", mid
    
def make_sha(password):
    import hashlib
    
    m = hashlib.sha1(password)
    return m.hexdigest()
    
def start_nyamuk(server, topic, name, username, password):
    ny = nyamuk.Nyamuk(name, logging.DEBUG)
    ny.on_message = on_message
    ny.on_connect = on_connect
    ny.on_subscribe = on_subscribe
    ny.keep_alive = 10
    the_topic = topic
    
    #rc = ny.connect(server, username = "satu", password = "satu")
    #rc = ny.connect(server)
    rc = ny.connect(server, username = username, password = make_sha(password))
    if rc != NC.ERR_SUCCESS:
        print "Can't connect"
        sys.exit(-1)
    
    index = 0
    while rc == NC.ERR_SUCCESS:
        rc = ny.loop()
        index += 1
        if index == 3:
            rc = ny.subscribe(topic, 0)
    
if __name__ == '__main__':
    print "len sys argv = ", len(sys.argv)
    if len(sys.argv) != 6:
        print "cara pakai : python pass-sub.py server topic name user pass"
        print "contoh     : python pass-sub.py localhost tekno paijo paijo paijo"
        sys.exit(0)
        
    server = sys.argv[1]
    topic = sys.argv[2]
    name = sys.argv[3]
    username = sys.argv[4]
    password = sys.argv[5]
    the_topic = topic
    start_nyamuk(server, topic, name, username, password)


'''
MQTT Constants
@author Iwan Budi Kusnanto
'''
UNKNOWN_VAL = -1

PROTOCOL_NAME = "MQIsdp"
PROTOCOL_VERSION = 3

#CLIENT_STATE
CS_NEW = 0
CS_CONNECTED = 1
CS_DISCONNECTING = 2

#socket
INVALID_SOCKET = -1

#keep alive timer, in seconds
KEEPALIVE_VAL = 120

#ERROR
ERR_SUCCESS = 0
ERR_NO_MEM = 1
ERR_PROTOCOL = 2
ERR_INVAL = 3
ERR_NO_CONN = 4
ERR_CONN_REFUSED = 5
ERR_NOT_FOUND = 6
ERR_CONN_LOST = 7
ERR_SSL = 8
ERR_PAYLOAD_SIZE = 9
ERR_NOT_SUPPORTED = 10
ERR_AUTH = 11
ERR_ACL_DENIED = 12
ERR_UNKNOWN = 13

#COMMAND
CMD_CONNECT = 0x10
CMD_CONNACK = 0x20
CMD_PUBLISH = 0x30
CMD_PUBACK = 0x40
CMD_PUBREC = 0x50
CMD_PUBREL = 0x60
CMD_PUBCOMP = 0x70
CMD_SUBSCRIBE = 0x80
CMD_SUBACK = 0x90
CMD_UNSUBSCRIBE = 0xA0
CMD_UNSUBACK = 0xB0
CMD_PINGREQ = 0xC0
CMD_PINGRESP = 0xD0
CMD_DISCONNECT = 0xE0

#OTHER
MESSAGE_RETRY = 20

#DIRECTION
DIRECTION_NONE = -1
DIRECTION_IN = 0
DIRECTION_OUT = 1

#MESSAGE STATE
MS_INVALID = 0
MS_WAIT_PUBACK = 1
MS_WAIT_PUBREC = 2
MS_WAIT_PUBREL = 3
MS_WAIT_PUBCOMP = 4

#CONNECT RETURN CODE
CONNECT_ACCEPTED = 0
CONNECT_RFSD_PROTO = 1
CONNECT_RFSD_ID = 2
CONNECT_RFSD_SERVER_UNAVAIL = 3
CONNECT_RFSD_BAD_USERPASS = 4
CONNECT_RFSD_NOT_AUTH = 5